/*
 * Copyright (c) 2022-present Charles7c Authors. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.zayac.admin.schedule;

import cn.hutool.core.util.StrUtil;
import com.zayac.admin.common.enums.DisEnableStatusEnum;
import com.zayac.admin.constant.ApiPathConstants;
import com.zayac.admin.req.MemberDetailsReq;
import com.zayac.admin.req.PayRecordsListReq;
import com.zayac.admin.req.team.TeamInfoReq;
import com.zayac.admin.req.team.TeamMemberListReq;
import com.zayac.admin.req.team.TeamMemberReq;
import com.zayac.admin.resp.*;
import com.zayac.admin.resp.team.Team;
import com.zayac.admin.resp.team.TeamAccount;
import com.zayac.admin.resp.team.TeamAccountWithChange;
import com.zayac.admin.resp.team.TeamMember;
import com.zayac.admin.service.CompletableFutureWebClientService;
import com.zayac.admin.service.TeamService;
import com.zayac.admin.service.TelegramMessageService;
import com.zayac.admin.system.model.entity.RoleDO;
import com.zayac.admin.system.model.entity.UserDO;
import com.zayac.admin.system.model.resp.AccountResp;
import com.zayac.admin.system.service.*;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import top.continew.starter.core.exception.BusinessException;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.temporal.TemporalAdjusters;
import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 定时任务 查询注册 新增用 暂定一分钟执行一次
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class TelegramTeamMessageSchedule {

    private final UserService userService;
    private final AccountService accountService;
    private final CompletableFutureWebClientService completableFutureWebClientService;
    private final TelegramMessageService telegramMessageService;
    private final TeamService teamService;
    private final RoleService roleService;
    private final UserRoleService userRoleService;

    private static final String MINISTER_ROLE_CODE = "minister";
    private static final long FIXED_DELAY = 60000L;

    @Scheduled(fixedDelay = FIXED_DELAY)
    public void newCheckRegistrationAndNewDeposit() {
        // Get the current date and time
        LocalDate nowDate = LocalDate.now();
        LocalDateTime nowDateTime = LocalDateTime.now();
        //获取部长角色
        RoleDO minister = roleService.getByCode(MINISTER_ROLE_CODE);
        List<Long> users = userRoleService.listUserIdByRoleId(minister.getId());
        //获取所有部长角色对应部门的用户
        users.forEach(userId -> {
            //传入对应部长用户
            processUser(userService.getById(userId), nowDate, nowDateTime).join();
        });
    }

    private CompletableFuture<Void> processUser(UserDO user, LocalDate nowDate, LocalDateTime nowDateTime) {
        //获取当前用户名下的所有团队账号
        List<AccountResp> accounts = accountService.getAccountsByUserId(user.getId(), DisEnableStatusEnum.ENABLE)
                .stream()
                .filter(AccountResp::getIsTeam)
                .toList();

        List<CompletableFuture<Void>> futures = accounts.stream()
                .map(account -> processTeamAccount(account, nowDate, nowDateTime))
                .toList();

        return CompletableFuture.allOf(futures.toArray(new CompletableFuture[0]));
    }

    private CompletableFuture<Void> processTeamAccount(AccountResp account,
                                                       LocalDate nowDate,
                                                       LocalDateTime nowDateTime) {
        TeamInfoReq teamInfoReq = TeamInfoReq.builder()
                .startDate(nowDateTime.with(TemporalAdjusters.firstDayOfMonth()))
                .endDate(nowDateTime)
                .build();
        CompletableFuture<Team> teamFuture = teamService.getLatestTeamInfoAsync(account, teamInfoReq);
        Team prevTeamInfo = teamService.getPreviousTeamInfo(account);

        return CompletableFuture.allOf(teamFuture).thenCompose(v -> {
            Team currentTeamInfo = teamFuture.join();
            log.info("Previous Team Info: {}", prevTeamInfo);
            log.info("Current Team Info: {}", currentTeamInfo);
            //获取代理线 代理对象映射
            Map<String, TeamAccount> teamAccountMap = currentTeamInfo.getList()
                    .stream()
                    .collect(Collectors.toMap(TeamAccount::getAgentName, Function.identity()));

            CompletableFuture<Void> registrationProcess = processRegistration(account, currentTeamInfo, prevTeamInfo, nowDate, teamAccountMap);
            CompletableFuture<Void> depositProcess = processDeposits(account, currentTeamInfo, prevTeamInfo, nowDate, nowDateTime);

            return CompletableFuture.allOf(registrationProcess, depositProcess)
                    .thenRun(() -> teamService.updateTeamInfo(account, currentTeamInfo));
        });
    }

    private CompletableFuture<Void> processRegistration(AccountResp account,
                                                        Team currentTeamInfo,
                                                        Team prevTeamInfo,
                                                        LocalDate nowDate,
                                                        Map<String, TeamAccount> teamAccountMap) {
        if (prevTeamInfo != null && currentTeamInfo.getSubMemberCount() > prevTeamInfo.getSubMemberCount()) {
            //代理线总共的新增注册数量
            int registerCount = currentTeamInfo.getSubMemberCount() - prevTeamInfo.getSubMemberCount();
            //构建查询参数
            TeamMemberReq memberListReq = TeamMemberReq.builder()
                    .registerStartDate(nowDate)
                    .registerEndDate(nowDate)
                    .startDate(nowDate)
                    .endDate(nowDate)
                    .registerSort(1)
                    .pageSize(registerCount)
                    .build();
            //查询所有新注册的会员
            CompletableFuture<MemberPagination<List<TeamMember>>> memberPaginationCompletableFuture = completableFutureWebClientService
                    .fetchDataForAccount(account, ApiPathConstants.MEMBER_TEAM_LIST_URL, memberListReq, new ParameterizedTypeReference<>() {
                    });
            return memberPaginationCompletableFuture.thenApply(MemberPagination::getList).thenAccept(members -> {
                log.info("Successfully get {} new registered members", members.size());
                //获取所有的新注册的会员对象
                if (!members.isEmpty()) {
                    //根据上级代理名分组
                    Map<String, List<TeamMember>> groupByTopAgentName = members.stream()
                            .collect(Collectors.groupingBy(TeamMember::getTopAgentName));
                    groupByTopAgentName.forEach((accountName, accountMembers) -> {
                                String memberNames = accountMembers.stream()
                                        .map(TeamMember::getName)
                                        .collect(Collectors.joining(", "));
                                log.info("Successfully get new registrations {}", memberNames);
                                String notification = String.format("👏 %s 注册: %d 用户: `%s` 总数:*%d*", accountName, accountMembers
                                        .size(), memberNames, teamAccountMap.get(accountName).getSubMemberNum());
                                UserDO currUser = accountService.getUserByAccountUsername(accountName);
                                //                                if (currUser != null && DisEnableStatusEnum.ENABLE.equals(currUser.getNeedNotify())) {
                                //                                    telegramMessageService.sendMessage(currUser.getBotToken(), currUser.getGroupId(), notification);
                                //                                }

                                telegramMessageService
                                        .sendMessage("6013830443:AAHUOS4v6Ln19ziZkH-L28-HZQLJrGcvhto", 6054562838L, notification);
                            }

                    );
                }
            });
        }
        return CompletableFuture.completedFuture(null);
    }

    private CompletableFuture<Void> processDeposits(AccountResp account,
                                                    Team currentTeam,
                                                    Team previousTeam,
                                                    LocalDate nowDate,
                                                    LocalDateTime nowDateTime) {
        return CompletableFuture.runAsync(() -> {
            if (previousTeam != null && currentTeam.getFirstDepositNum() > previousTeam.getFirstDepositNum()) {
                PayRecordsListReq req = PayRecordsListReq.builder()
                        .startDate(nowDate)
                        .endDate(nowDate)
                        .pageSize(100)
                        .payState(2)
                        .build();
                CompletableFuture<Pagination<List<PayRecord>>> paginationCompletableFuture = completableFutureWebClientService
                        .fetchDataForAccount(account, ApiPathConstants.PAY_RECORDS_LIST_URL, req, new ParameterizedTypeReference<>() {
                        });
                List<TeamAccountWithChange> changedTeamAccounts = findChangedTeamAccount(previousTeam, currentTeam);
                Set<String> changedAgentNames = changedTeamAccounts.stream()
                        .filter(teamAccountWithChange -> teamAccountWithChange.getNewDepositNum() > 0)
                        .map(TeamAccountWithChange::getAgentName)
                        .collect(Collectors.toSet());
                paginationCompletableFuture.thenApply(Pagination::getList).thenAccept(payRecords -> {

                    Map<String, List<String>> agentNameWithNames = payRecords.stream()
                            .filter(record -> record.getCreatedAt().isAfter(nowDateTime.minusHours(1)) && changedAgentNames.contains(record.getAgentName()))
                            .collect(Collectors.groupingBy(PayRecord::getAgentName, Collectors
                                    .mapping(PayRecord::getName, Collectors.collectingAndThen(Collectors.toSet(), ArrayList::new))));

                    agentNameWithNames.forEach((agentName, names) -> {
                        StringBuilder depositResults = new StringBuilder();
                        AtomicInteger depositCounter = new AtomicInteger(0);

                        TeamAccountWithChange targetTeamAccount = changedTeamAccounts.stream()
                                .filter(teamAccount -> StrUtil.equals(teamAccount.getAgentName(), agentName))
                                .findFirst()
                                .orElseThrow(() -> new BusinessException(String.format("can not find agent name %s", agentName)));

                        List<CompletableFuture<Void>> fetchFutures = names.stream()
                                .map(name -> fetchMemberDetails(account, name, nowDate).thenAccept(member -> {
                                    payRecords.stream()
                                            .filter(record -> record.getCreatedAt().equals(member.getFirstPayAt()))
                                            .findFirst()
                                            .ifPresent(record -> {
                                                if (depositCounter.getAndIncrement() < targetTeamAccount.getNewDepositNum()) {
                                                    depositResults.append(String.format("用户: `%s`, 首存金额: *%s*\n", member.getName(), record.getScoreAmount()));
                                                }
                                            });
                                }).exceptionally(ex -> {
                                    log.error("Error fetching details for member {}: {}", name, ex.getMessage());
                                    return null;
                                }))
                                .toList();

                        CompletableFuture<Void> allFetches = CompletableFuture.allOf(fetchFutures.toArray(new CompletableFuture[0]));

                        allFetches.thenRun(() -> {
                            if (!depositResults.isEmpty()) {
                                String notification = String.format("🎉 %s 首存: *%d* %s 总数: *%d*", agentName, targetTeamAccount.getNewDepositNum(), depositResults, targetTeamAccount.getFirstDepositNum());

                                UserDO currUser = accountService.getUserByAccountUsername(agentName);
//                            if (currUser != null && DisEnableStatusEnum.ENABLE.equals(currUser.getNeedNotify())) {
//                                telegramMessageService.sendMessage(currUser.getBotToken(), currUser.getGroupId(), depResults);
//                            }
                                telegramMessageService.sendMessage("6013830443:AAHUOS4v6Ln19ziZkH-L28-HZQLJrGcvhto", 6054562838L, notification);
                            }
                        }).exceptionally(ex -> {
                            log.error("Error sending notification for account {}: {}", account.getId(), ex.getMessage());
                            return null;
                        });
                    });
                }).exceptionally(ex -> {
                    log.error("Error processing deposits for account {}: {}", account.getId(), ex.getMessage());
                    return null;
                }).join();
            }
        });
    }


    //根据用户名查询对应会员详情
    private CompletableFuture<Member> fetchMemberDetails(AccountResp account, String name, LocalDate nowDate) {
        TeamMemberListReq memberListReq = TeamMemberListReq.builder()
                .name(name)
                .startDate(nowDate)
                .endDate(nowDate)
                .status(1)
                .build();

        CompletableFuture<MemberPagination<List<Member>>> memberFuture = completableFutureWebClientService
                .fetchDataForAccount(account, ApiPathConstants.MEMBER_TEAM_LIST_URL, memberListReq, new ParameterizedTypeReference<>() {
                });

        return memberFuture.thenApply(MemberPagination::getList)
                .thenApply(list -> list.stream().findFirst())
                .thenCompose(optionalMember -> optionalMember.map(member -> fetchDetailedMemberInfo(account, member
                        .getId(), nowDate)).orElseThrow(() -> new RuntimeException("没有找到匹配的成员信息")));
    }

    private CompletableFuture<Member> fetchDetailedMemberInfo(AccountResp account, Long memberId, LocalDate nowDate) {
        MemberDetailsReq detailsReq = MemberDetailsReq.builder()
                .id(memberId)
                .startDate(nowDate)
                .endDate(nowDate)
                .build();

        return completableFutureWebClientService
                .fetchDataForAccount(account, ApiPathConstants.MEMBER_DETAIL_URL, detailsReq, new ParameterizedTypeReference<>() {
                });
    }

    /**
     * 对比两个Team中发生变化的具体TeamAccount
     *
     * @param prevTeam 缓存中的Team
     * @param currTeam 新查询到的Team
     * @return TeamAccountWithChange
     */
    public List<TeamAccountWithChange> findChangedTeamAccount(Team prevTeam, Team currTeam) {
        Map<Long, TeamAccount> team2AccountMap = currTeam.getList()
                .stream()
                .collect(Collectors.toMap(TeamAccount::getId, account -> account));

        return prevTeam.getList()
                .stream()
                .filter(account1 -> team2AccountMap.containsKey(account1.getId()))
                .map(account1 -> {
                    TeamAccount account2 = team2AccountMap.get(account1.getId());
                    TeamAccountWithChange changedAccount = new TeamAccountWithChange();
                    BeanUtils.copyProperties(account2, changedAccount);
                    if (account1.getFirstDepositNum() != account2.getFirstDepositNum()) {
                        changedAccount.setNewDepositNum(account2.getFirstDepositNum() - account1.getFirstDepositNum());
                    }
                    if (account1.getSubMemberNum() != account2.getSubMemberNum()) {
                        changedAccount.setNewRegisterNum(account2.getSubMemberNum() - account1.getSubMemberNum());
                    }
                    return changedAccount;
                })
                .collect(Collectors.toList());
    }
}
