package com.zayac.admin.service;

import cn.hutool.core.lang.TypeReference;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.zayac.admin.resp.ApiResponse;
import com.zayac.admin.system.model.resp.AccountResp;
import com.zayac.admin.system.service.AccountService;
import io.netty.handler.timeout.TimeoutException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.http.HttpStatusCode;
import org.springframework.stereotype.Service;
import org.springframework.web.reactive.function.BodyInserters;
import org.springframework.web.reactive.function.client.WebClient;
import org.springframework.web.reactive.function.client.WebClientResponseException;
import reactor.core.publisher.Mono;
import reactor.core.scheduler.Schedulers;
import reactor.util.retry.Retry;
import top.continew.starter.core.exception.BusinessException;

import java.time.Duration;
import java.util.Map;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.Semaphore;
import java.util.concurrent.TimeUnit;

@Service
@Slf4j
public class CompletableFutureWebClientService {
    private final WebClient webClient;
    private final Semaphore semaphore;
    private final ObjectMapper objectMapper;
    private final LoginService loginService;
    private final AccountService accountService;

    public CompletableFutureWebClientService(WebClient.Builder webClientBuilder,
                                             @Value("${webclient.max-concurrent-requests}") int maxConcurrentRequests,
                                             ObjectMapper objectMapper,
                                             LoginService loginService,
                                             AccountService accountService) {
        this.webClient = webClientBuilder.build();
        this.semaphore = new Semaphore(maxConcurrentRequests);
        this.objectMapper = objectMapper;
        this.loginService = loginService;
        this.accountService = accountService;
    }

    public <T> CompletableFuture<T> fetchDataForAccount(AccountResp account,
                                                        String apiPath,
                                                        Object params,
                                                        ParameterizedTypeReference<ApiResponse<T>> typeRef) {
        return ensureLoggedIn(account)
                .thenCompose(ignored -> fetchData(account.getPlatformUrl() + apiPath, account.getHeaders(), params, typeRef, account).toFuture());
    }

    private CompletableFuture<Void> ensureLoggedIn(AccountResp account) {
        if (StrUtil.isEmptyIfStr(account.getHeaders())) {
            return loginService.reLoginAndGetHeaders(account)
                    .flatMap(newHeaders -> {
                        account.setHeaders(newHeaders);
                        accountService.updateHeaders(newHeaders, account.getId());
                        return Mono.empty();
                    })
                    .then()
                    .toFuture();
        } else {
            return CompletableFuture.completedFuture(null);
        }
    }

    public <T> Mono<T> fetchData(String url,
                                 String headers,
                                 Object params,
                                 ParameterizedTypeReference<ApiResponse<T>> typeRef,
                                 AccountResp account) {
        return Mono.fromCallable(() -> {
            if (!semaphore.tryAcquire(10, TimeUnit.SECONDS)) {
                throw new RuntimeException("Unable to acquire a permit");
            }
            return true;
        }).subscribeOn(Schedulers.boundedElastic()).then(
                this.webClient.post().uri(url).headers(httpHeaders -> {
                            try {
                                Map<String, String> headerMap = JSONUtil.toBean(headers, new TypeReference<>() {
                                }, true);
                                headerMap.forEach(httpHeaders::add);
                            } catch (Exception e) {
                                log.warn("Header conversion exception: " + e.getMessage());
                                throw new BusinessException("Header conversion failed", e);
                            }
                        })
                        .body(params != null ? BodyInserters.fromValue(params) : BodyInserters.empty())
                        .retrieve()
                        .onStatus(HttpStatusCode::isError, response -> Mono.error(new BusinessException("API call failed")))
                        .bodyToMono(String.class)
                        .doOnNext(resStr -> {
                            log.info("request url:{}", url);
                            log.info("request headers :{}", headers);
                            log.info("request params:{}", params);
                            log.info("response {}", resStr);
                        })
                        .flatMap(body -> {
                            try {
                                ApiResponse<T> apiResponse = objectMapper.readValue(body, objectMapper.getTypeFactory()
                                        .constructType(typeRef.getType()));
                                return Mono.justOrEmpty(apiResponse);
                            } catch (Exception e) {
                                log.warn("JSON parsing exception: " + e.getMessage());
                                return Mono.just(new ApiResponse<T>(null, "Decoding error", 6008));
                            }
                        })
                        .flatMap(response -> respHandler(response, url, params, typeRef, account))
                        .retryWhen(Retry.backoff(3, Duration.ofSeconds(3)).filter(this::isRetryableException))
        ).doFinally(signal -> semaphore.release());
    }

    private boolean isRetryableException(Throwable throwable) {
        return throwable instanceof TimeoutException || throwable instanceof WebClientResponseException.ServiceUnavailable;
    }

    private <T> Mono<T> respHandler(ApiResponse<T> response, String url, Object params,
                                    ParameterizedTypeReference<ApiResponse<T>> typeRef, AccountResp account) {
        if (response.getStatusCode().equals(6000)) {
            return Mono.just(response.getData());
        } else if (response.getStatusCode().equals(6001)) {
            return loginService.reLoginAndGetHeaders(account)
                    .flatMap(newHeaders -> {
                        // 更新 account 对象中的 headers
                        account.setHeaders(newHeaders);
                        accountService.updateHeaders(newHeaders, account.getId());
                        // 重新发送原始请求
                        return fetchData(url, newHeaders, params, typeRef, account)
                                .doOnNext(data -> log.info("Retried request successful"));
                    })
                    .onErrorResume(e -> Mono.error(new BusinessException("Re-login failed: " + e.getMessage())));
        } else {
            return Mono.error(new BusinessException("Error status code: " + response.getStatusCode()));
        }
    }
}
